// This file is part of GNOME Games. License: GPL-3.0+.

private errordomain Games.RetroError {
	MODULE_NOT_FOUND,
	INVALID_GAME_FILE,
	COULDNT_WRITE_SNAPSHOT,
	COULDNT_LOAD_SNAPSHOT,
	FIRMWARE_NOT_FOUND,
}
